from fasthtml.fastapp import FastHTML
from starlette import status
from starlette.responses import StreamingResponse
from starlette.exceptions import HTTPException
from pydantic import ValidationError

from .utils import load_database, save_database, generate_data
from .custom_types import TestDataBody

def create_api(api_keys: set[str], max_size: int, max_data: int, database: str, buffer_size: int) -> FastHTML:

    api = FastHTML()

    class MaxSizePerRequestError(Exception):
        pass


    class MinSizePerRequestError(Exception):
        pass

    @api.get('/')  # type: ignore
    async def test_data(api_key: str, size: str) -> StreamingResponse:
        try:
            body = TestDataBody(api_key=api_key, size=size)  # type: ignore
        except ValidationError as err:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail='Invalid Format.'
            ) from err

        try:
            if body.api_key not in api_keys:
                raise HTTPException(
                    status_code=status.HTTP_401_UNAUTHORIZED,
                    detail='Invalid API Key.'
                )

            if body.size < 0:
                raise MinSizePerRequestError

            if max_size < body.size:
                raise MaxSizePerRequestError

            db = load_database(database)
            if max_data <= db['data-used'] + body.size:
                raise HTTPException(
                    status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                    detail='Service not available.'
                )
            db['data-used'] += body.size

            save_database(database, db)

            return StreamingResponse(
                status_code=status.HTTP_200_OK,
                content=generate_data(body.size, buffer_size),
                media_type='application/octet-stream',
                headers={
                    'Content-Length': str(body.size)
                }
            )

        except MinSizePerRequestError as err:
            raise HTTPException(
                status_code=status.HTTP_416_REQUESTED_RANGE_NOT_SATISFIABLE,
                detail='Size has to be not-negative.'
            ) from err
        except MaxSizePerRequestError as err:
            raise HTTPException(
                status_code=status.HTTP_416_REQUESTED_RANGE_NOT_SATISFIABLE,
                detail=f'Exceeded max size per request of {max_size} Bytes.'
            ) from err

    return api
